from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import Select
import time
import re
from bs4 import BeautifulSoup


# ICI nous renseignons les options de notre navigateur
chrome_options = webdriver.ChromeOptions()
# dans notre service nous voulons lancer le chrome sans interface (options headless)
# en cas de lancement sur un poste personnel il est possible d'enlever ce paramètre.
chrome_options.add_argument('--headless')




def remplir_destination(browser,destination):
    destination_select = Select(browser.find_element(By.CLASS_NAME, "select_destination_moteur_home"))
    destination_select.select_by_value(destination)

def refuser_cookies(browser):
    refuser_cookies = browser.find_element(By.ID, "cookieconsent-decline")
    #accepter_cookies = browser.find_element(By.ID, "cookieconsent-accept")
    refuser_cookies.click()
    browser.save_screenshot('apres_refus_cookies.png')


def remplir_date_arrivee(browser, jour_arrivee):
    ## ce code ne marche que pour les jours du mois courant... si on veut changer de mois il faut
    ## repérer le bouton mois suivant, clicker et ensuite aller chercher le jour
    # ici on click sur le champ date de départ, le calendrier s'ouvre.
    button_checkin = browser.find_element(By.ID, "check1")
    button_checkin.click()
    # pour repérer des éléments, nous pouvons également utiliser XPATH
    # c'est un langage permettant de trouver le chemin d'éléments XML (language proche du HTML)
    # https://www.w3schools.com/xml/xpath_intro.asp
    # ex : jourDepart = browser.find_element(By.XPATH,"//*[text()='"+jour_depart+"']")
    # le problème est que parfois on va avoir plusieurs éléments avec le meme texte, un prix d'hotel
    # peut être identique à la date.
    # je propose donc la solution ci dessous, pour se restreindre d'abord aux éléments <a> : 

    ancres = browser.find_elements(By.TAG_NAME, "a")
    jourArrivee = None
    # et ensuite on retiens seulement l'élément avec comme texte notre jour de départ
    for ancre in ancres:
        if ancre.text == jour_arrivee:
            jourArrivee = ancre
    jourArrivee.click()



def remplir_date_depart(browser, jour_depart):
    # pour le départ, pas besoin d'ouvrir le calendrier ,
    # il s'est ouvert tout seul à l'étape précédente
    ancres = browser.find_elements(By.TAG_NAME, "a")
    jourDepart=None
    # et ensuite on retiens seulement l'élément avec comme texte notre jour de départ
    for ancre in ancres:
        if ancre.text == jour_depart:
            jourDepart = ancre
    jourDepart.click()


def lancer_recherche(browser):
    rechercher = browser.find_element(By.ID, "boutonr")
    rechercher.click()  
# ci dessous une fonction pour récupèrer
#  les différentes informations à partir de l'objet soup d'un  hotel
# à améliorer pour prendre en compte lorsque l'on n'arrive pas à trouver certains éléments et 
# gérer les erreurs / exceptions
# c'est une première version, le nettoyage des champs n'est pas abouti


def recuperer_infos_cartes_hotels(hotels):
    hotels_Bs = []
    ## on peut ensuite utiliser beautifulSoup si l'on souhaite, pour ce faire reste à récuperer le HTML
    for hotel in hotels:
        html = hotel.get_attribute('innerHTML')
        f = open("code_html_hotel.txt", "w", encoding="utf-8")
        f.write(html)
        f.close()
        soup = BeautifulSoup(html, 'html.parser')
        infos_hotel = recuperer_infos_carte_hotel(soup)
        hotels_Bs.append(infos_hotel)
    return hotels_Bs


def recuperer_infos_carte_hotel(soup_hotel):
    nomHotel = soup_hotel.find('span', attrs={'class':'tittre_hotel'}).text
    icones_etoile = soup_hotel.find('div', attrs={'class' : 'etoilessa'}).find_all('i')
    prix = soup_hotel.find('div', attrs={'class': 'monaieprix'}).text
    return {'prix': prix, 'nom': nomHotel, 'nombre_etoiles': len(icones_etoile) }

# la fonction pour effectuer
# la collecte des informations


def rechercherUnSejour(destination,  jour_arrivee, jour_depart):
    browser = webdriver.Chrome(options=chrome_options)
    browser.get('https://www.tunisiebooking.com')
    browser.save_screenshot('apres_url_booking.png')
    # nous pouvons rajouter des temps d'attente, pour être sûr 
    # que les éléments ont bien eu le temps de s'afficher après le click.
    time.sleep(2)
    remplir_destination(browser, destination)
    # il faut accepter ou refuser les cookies sinon on ne peut pas clicker sur le date picker
    time.sleep(2)
    refuser_cookies(browser)
    time.sleep(2)
    remplir_date_arrivee(browser, jour_arrivee)
    time.sleep(2)
    remplir_date_depart(browser, jour_depart)
    time.sleep(2)
    lancer_recherche(browser)
    time.sleep(2)
    browser.save_screenshot('apres_recherche.png')
    # nous sommes sur la nouvelle page, on peut par exemple récupérer une liste de produits
    hotels = browser.find_elements(By.XPATH, '//div[starts-with(@id, "produit_affair_")]')
    # si on veut afficher le nombre d'hotels trouvé pour comparer avec ce qui est affiché à l'écran : 
    print("nombre d'hôtels sur la page renvoyée par la recherche : " + str(len(hotels)))

    hotels_Bs = recuperer_infos_cartes_hotels(hotels)
    # on affiche le résultat, idéalement on le stockerai quelque part (fichier, base)
    # soit depuis cette fonction, soit en appelant une autre fonction
    print(hotels_Bs)
    # ne pas oublier de ferme le navigateur une fois que l'on a terminé toutes nos requêtes
    browser.quit()


rechercherUnSejour("Monastir", "27", "29")




#if you want to print the content of the page into a log file at any time:
#code_source=browser.page_source.encode("utf-8")
#with open('logs.txt', 'w') as f:
#    f.write(str(code_source))