# Le web scraping

---

## La démarche :

- Caractériser son objectif
- Analyse et repérage du site cible et d'où sont situées les informations
- réalisation du code de collecte
- mise en forme du code pour structurer un résultat.
- externaliser les variables d'environnement et le stockage des résultats 

---

Plusieurs librairies pour avoir un client http en python:

- bas niveau (pas fait pour être utilisé directement)
    -  http.client 
- haut niveau:
    - requests => celui qu'on va utiliser
    - urllib3
    - aiohttp

---

## Le module requests

documentation officielle bien faite : https://requests.readthedocs.io/en/latest/.

Le module permet d'envoyer des requetes http.


---

## Le module requests

La fonction get permet ainsi d'envoyer des requetes de type get.

Des paramètres usuels:

- url
- proxies
- headers




---

## 1ere étape : récupérer le code HTML de la page cible

avec une requete de type get :

```python 
response = requests.get(url, proxies, headers)
```

avec une requete de type post :
```python 
response = requests.post(url, proxies, headers, data)
```

---

### Le web scraping  avec BeautifullSoup  1/3



Beautiful Soup pertmet d'avoir un objet BeautifulSoup à partir d'uns code HTML.

L'objet aura une structure hiérarchique (parent/enfant) et des méthodes de la classe nous permettrons de parcourir plus facilement les informations.
```
soup=BeautifulSoup(response.text, 'html.parser')
```
- il existe deux principales librairies pour parcourir ou parser du HTML en python: lxml et HTML Parser

---

### Le web scraping  avec BeautifullSoup  2/3

Les attributs des objets la classe BeautifulSoup vont permettre de parcourir cette structure de données

- title
- les elements HTML(p, div, a...)
- text : permet de se débarasser du HTML

---

### Le web scraping  avec BeautifullSoup  3/3

Les méthodes de la classe BeautifulSoup:
trouver des éléments HTML, avec éventuellement des conditions sur
leurs attributs

- find : renvoie le premier éléments trouvé
```
soup.find("tr", attrs={"class":"Entete1"})
```
- findAll : renvoie la liste des éléments vérifiant la condition précisée.
```
## [0] permet de prendre le premier élément pour se ramener au résultat de la méthode find dans cet exemple
soup.find_all("tr", attrs={"class":"Entete1"})[0]
```

---

### Les sites se protègent 

Attention:

- Afin d’éviter la réutilisation de leurs contenus par des concurrents et lutter contre l’espionnage
- Afin de bloquer les bots qui occupent une partie significative du trafic, ralentissant ainsi l’accès au site
    - Un scraping mal fait peut s’assimiler à une attaque par déni de service (DoS)

---

### Quelques pratiques admises pour ne pas être bloqué par les sites scrapés :

- Ajouter des pauses entre chaque requête, pour ne pas aller plus vite qu’un utilisateur ne le pourrait (time.sleep() en Python)
    - C’est également une bonne pratique de scraping éthique
    - Un délai de 2 à 5 secondes par requête est la préconisation usuelle
- Modifier régulièrement le champ user-agent (par exemple avec un compteur) avant que celui-ci ne se fasse bloquer

---

### Autres recommandations pour un INS

- Faire en sorte de pouvoir tracer l’activité d’un programme, afin de notamment pouvoir en auditer le fonctionnement.
    - Les logs doivent contenir des informations sur l’utilisateur ou le compte système effectuant l’action, le résultat de l’action et le temps qu’elle a mis à s’exécuter.
- En cas de passage en production, il est nécessaire que le programme fasse l’objet d’une revue de code (entre pairs ou externe), voire, selon la nature de son intégration, relève d’une démarche d’homologation de sécurité.

---

### Autres recommandations pour un INS

- Bien faire attention à la documentation et à la transmission des codes lors de projets de scraping, pour éviter les pertes de connaissance au sein d’une équipe lors d’un changement de poste.
    - Il est déjà arrivé que des collectes soient freinées en raison du départ de la personne à les avoir mises en place.

