
### Selenium

- BeautifulSoup sera suffisant quand vous voudrez travailler sur des pages HTML statiques. 
- Dès que les informations que vous recherchez sont générées via l’exécution de scripts Javascript, il vous faudra passer par des outils comme Selenium.
- Nous avons fait des petites experimentations ici:
https://git.lab.sspcloud.fr/methodo-ipc/webscraping-with-selenium

---

## Principe:
 
- Selenium est un outil developpé initialement pour faire des tests sur navigateurs.
- Il existe d'autres outils similaires (Cypress par exemple)
- Il a l'avantage d'avoir des déclinaisons sur plusieurs langages (Java, Python, php ...)
- Mimer le comportement d'un utilisateur sur un site web, sur un navigateur.
- https://pypi.org/project/selenium/


---

## Installation en python :

### poste personnel / de travail 1/2

- Il faut avoir un navigateur d'installer sur le poste
- télécharger un driver pour notre navigateur.
(regarder celui correspondant à votre navigateur dans la doc https://pypi.org/project/selenium/)
- choisir un .zip et le télécharger


---

## Installation en python :
### poste personnel / de travail 2/2


- dezipper le driver
- rajouter un lien dans le path vers le .exe du driver
(sur windows, Rechercher, modifier les variables d'environnement pour votre compte, modifier la variable path en rajoutant une valeur )

---


## Installation en python :
### environnement à la demande (service sspCloud par ex) 1/2


- pour installer un navigateur dans un service SSPCloud, les commandes à lancer dans un terminal sont :

```
wget https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb -O /tmp/chrome.deb
sudo apt-get update
sudo apt-get install -y /tmp/chrome.deb
```

---

## Installation en python :
### environnement à la demande (service sspCloud par ex) 2/2

Il faudra ensuite lancer une fois les commandes ci dessous pour installer le driver:
```
pip install chromedriver-autoinstaller selenium
import chromedriver_autoinstaller
chromedriver_autoinstaller.install()
```

---


## lancer le navigateur

```python
from selenium import webdriver
# ICI nous renseignons les options de notre navigateur
chrome_options = webdriver.ChromeOptions()
# dans notre service nous voulons lancer le chrome sans interface (options headless)
# en cas de lancement sur un poste personnel il est possible d'enlever ce paramètre.
# dans ce cas, le navigateur s'ouvrira également et nous pourrons voir le contenu parcouru
chrome_options.add_argument('--headless')
browser = webdriver.Chrome(options=chrome_options)
```

---

## Repérer les éléments 1/2

- Le repérage des éléments se fait d'une manière similaire qu'avec requests (id, className,...)
- deux méthodes principales : find_element et find_elements (renvoie une liste)
- après le code de la diapo précédente on peut ajouter : 
```python
from selenium.webdriver.common.by import By
close_button = browser.find_element(By.ID, "close-button")
```
https://selenium-python.readthedocs.io/locating-elements.html

---

## Repérer les éléments 2/2 

il y a les conditions sur les attributs classique:

- By.ID
- By.NAME
- By.CLASS_NAME

et puis la possibilité d'utiliser des selecteurs particuliers pour les cas les plus complexes
- By.XPATH
- By.CSS_SELECTOR 



---

## on peut remplir des formulaires et vérifier le contenu

```python
quantity = browser.find_element(By.NAME, "websitebundle_quotation_type[quantity]")

quantity.clear()
quantity.send_keys(1000)

element_attribute_value = quantity.get_attribute('value')
```

---

## on peut clicker sur des boutons

```
button = browser.find_element("name", "websitebundle_quotation_type[submit]") 
button.click()
```


---

## on peut faire des captures d'écrans au fur et à mesure

```python
browser.save_screenshot('apres_clic.png')
```

