# Mise en pratique avec requests et BeautifulSoup

---

## La démarche :

- regarder en détail le contenu HTML de la page du site qui nous intéresse.
- regarder les requêtes qui sont envoyées dans l'onglet réseau (url et méthode)
- notre 1er site d'intéret : http://www.tunisie-annonce.com/
- nous voulons nous intéresser dans cet exercice à l'immobilier

---

## Utilisons l'inspecteur et la console
DEMO

---

## les urls 
- sans filtre / avec la methode get : 
http://www.tunisie-annonce.com/AnnoncesImmobilier.asp

- avec filtre, avec la methode post : http://www.tunisie-annonce.com/AnnoncesImmobilier.asp?rech_order_by=31

- `la méthode post n'a pas l'air d'être utilisée à bon essient, c'est plutôt pour les formulaire ou envoyer des donner que filtrer`

---

## Pratique : faire la requete HTTP


- écrire le code pour faire la requête get à l'url 
http://www.tunisie-annonce.com/AnnoncesImmobilier.asp
- écrirer le code pour faire la requête post avec paramètres.

---

## Solution : requête get 

```python
# il faut bien définir les headers et éventuellement le proxy
url='http://www.tunisie-annonce.com/AnnoncesImmobilier.asp'
proxyDict = None
userAgent = '''INSEE (statistique publique) - 
collecte expérimentale sans but de diffusion - 
Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) 
Gecko/20100101 Firefox/115.0'''
acceptLanguage = 'fr'
headers = {"user-agent": userAgent, "accept-language": acceptLanguage}
response = requests.get(url, proxies=proxyDict, headers=headers)
```

---

## Solution : requête post 

```python
# il faut bien définir les headers et éventuellement le proxy
url='http://www.tunisie-annonce.com/AnnoncesImmobilier.asp'
proxyDict = None
userAgent = '''INSEE (statistique publique) - collecte expérimentale sans but de diffusion - 
Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0'''
acceptLanguage = 'fr'
headers = {"user-agent": userAgent, "accept-language": acceptLanguage}
myobj = {
	"rech_cod_pay": "TN",
	"rech_cod_reg": "104"
}
response = requests.post(url, proxies=proxyDict, headers=headers, data=myObj)
```

---

## Pratique : créer un objet BeautifulSoup


- écrire le code permettant de créer un objet BeautifulSoup pour exploiter le contenu de la page HTML.

---

## Solution : créer un objet BeautifulSoup

```python
soup = BeautifulSoup(response.text, 'html.parser')
```

---

## Pratique : extraire le nombre d'annonces

Cherchons tout d'abord le nombre d'annonces.

- Selon vous, quel element nous permettra d'extraire cette information?
- une fois l'élement trouvé, écrivez le code permettant de faire cette extraction.

---

## Solution

```HTML
    <td colspan="13">
        <table class="RecordsNumber" width="100%" cellspacing="0" cellpadding="0">
        <tbody><tr>
        <td>
            Résultat de votre recherche :&nbsp;<b>20 947 annonces</b>
        </td>
            ...
        </tr>
        </tbody></table>
    </td>
 ```


```python
def extraire_annonces(soup):
	tableau=soup.find("table",attrs={"class":"RecordsNumber"})
	texte=tableau.find("b").text
	resultat=0
	if "annonces" in texte:
    		resultat=re.find_all(r'\d+', texte)[0]
	return resultat
```


---

## Reflexions à avoir en pratique

La manière dont on construit le code va impacter
- sa maintenabilité
- sa robustesse:
    - identifier les variables indispensables
    - mettre des controles / traiter les exceptions




---

## Le tableau des annonces

Plan d'attaque: 
- Trouver le tableau
- récupérer l'entête
- Récupérer chaque ligne du tableau
- Nettoyer les sorties en ne gardant que le texte sur une ligne
- Finalisation du tableau
- Généraliser sur toutes les pages?

---

## pratique  : Le tableau des annonces
Récupérons la liste des variables

- Selon vous, quel element nous permettra d'extraire cette information?
- une fois l'élement trouvé, écrivez le code permettant de faire cette extraction.


---

Le code HTML de la page : 
```HTML
<tr class="Entete1" valign="middle" bgcolor="#294a73">
	<td width="1" bgcolor="#294a73"></td>
	<td width="18%">&nbsp;Région</td><td width="1" bgcolor="white"></td>
	<td width="18%">&nbsp;Nature</td><td width="1" bgcolor="white"></td>
	<td width="13%">&nbsp;Type</td><td width="1" bgcolor="white"></td>
	<td width="28%">&nbsp;Texte annonce</td><td width="1" bgcolor="white"></td>
	<td width="11%"><table width="100%" border="0"><tbody><tr><td>&nbsp;<b>Prix</b></td></tr></tbody></table></td><td width="1" bgcolor="white"></td>
	<td width="12%"><table width="100%" border="0"><tbody><tr><td>&nbsp;<b>Modifiée</b></td><td height="8"><img src="/images/n_Fleche_bas_active.gif" alt="" width="7" height="8"></td></tr></tbody></table></td><td width="1" bgcolor="#294a73"></td>
</tr>
```

Solution en python (1ere version):
```python
def extraire_nom_colonnes(soup):
    headers=soup.find("tr", attrs={"class":"Entete1"})
	noms_colonnes=headers.find_all("td")
	colonnes=[]
	for n in noms_colonnes:
		colonnes.append(n.text)
	return(colonnes)
```
Mais il y a des elements </ td> qui contiennent aucun texte ...

---

Solution en python (2e version):
```python
def extraire_nom_colonnes(soup):
    headers=soup.find("tr", attrs={"class":"Entete1"})
	noms_colonnes=headers.find_all("td")
	colonnes=[]
	for n in noms_colonnes:
		if n.text!='' and n.text not in colonnes:
			colonnes.append(n.text)
	return(colonnes)
```

---

Récupérons les données du tableau de la 1ere page:

```python
def extraire_donnees_tableau(soup,noms_colonnes):
	liste_lignes = soup.find_all("tr", attrs={"class":"Tableau1"})
	print("nombre de lignes :"+ str(len(liste_lignes)))
	table_page1 = []
	for l in liste_lignes:
		donnees=l.find_all("td")
		a=0
		dic={}
		for j in range(len(donnees)):
			if(donnees[j].text):
				dic[noms_colonnes[a]]=donnees[j].text
				a=a+1
		table_page1.append(dic)
	df=pd.DataFrame(table_page1,columns =noms_colonnes)
	print("donnees du tableau : ")
	print(df)
	df.to_csv('resultat.csv')
	return(df)
```


---

Remarque:
- nous avons rempli les colonnes dans l'ordre, n'ayant pas d'information
au niveau des élements < td> sur la variable qu'ils contiennent.
- si des champs peuvent être nuls, il y a un risque de décalage...

